@@ -1,5 +1,5 @@
 ;;; jde-dbo.el -- JDEbug output functions
-;; $Revision: 1.5 $ $Date: 2002/09/14 03:20:55 $ 
+;; $Revision: 1.39 $ $Date: 2003/09/27 05:34:36 $ 
 
 ;; Author: Paul Kinnucan <paulk@mathworks.com>
 ;; Maintainer: Paul Kinnucan
@@ -136,7 +136,20 @@ exception spec."
 	    (oset jde-dbs-the-process-registry :target-process process)))
       (message "Start Event Error: can't find process object for process id %d" process-id))))
 
-
+(defun jde-dbo-break (process state-info state reason thread-id thread-name
+                              message proc-id class file line-no)
+  (jde-dbs-proc-state-info-set state-info state reason
+                               thread-id thread-name)
+  (setq jde-dbo-current-process process)
+  (setq jde-dbo-current-thread-id thread-id)
+  (if jde-bug-local-variables
+    (jde-dbo-update-locals-buf process
+                               thread-id 0))
+  (if jde-bug-stack-info (jde-dbo-update-stack process thread-id))
+  (oset process steppablep t)
+  (jde-dbs-display-debug-message proc-id message)
+  (jde-db-set-debug-cursor class file line-no)
+  (when jde-bug-raise-frame-p (raise-frame)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;                                                                            ;; 
@@ -211,55 +224,10 @@ See also the hook `tree-widget-after-toggle-fucntions'."
                              :object-id (oref this-obj :id))))))
         
       ;; Insert the local variables for this stack frame.
-      (while locals
-	(setq var (car locals) locals (cdr locals))
-	(let* ((var-name (oref var name))
-	       (var-type (oref var jtype))
-	       (var-value (oref var value))
-	       (var-tag (format "%s %s" var-type var-name)))
-	  (cond
-	   ((typep var-value 'jde-dbs-java-udci)
-	    (if (string= (oref var-value :jtype) "java.lang.String")
-		(let* ((cmd (jde-dbs-get-string 
-				   "get string"
-				   :process process
-				   :object-id (oref var-value id)))
-		       (str-val (jde-dbs-cmd-exec cmd)))
-		  (widget-create 'tree-widget
-                                 :tag var-tag
-                                 :node-name var-tag
-                                 :open (jde-dbo-locals-open-p var-tag)
-                                 :value t
-				 (list 'tree-widget :tag str-val)))
-	      (widget-create 'jde-widget-java-obj
-                             :tag var-tag
-                             :node-name var-tag
-                             :open (jde-dbo-locals-open-p var-tag)
-			     :process process
-                             :object-id (oref var-value :id))))
-	   ((typep var-value 'jde-dbs-java-array)
-	    (widget-create 'jde-widget-java-array
-                           :tag var-tag
-                           :node-name var-tag
-                           :open (jde-dbo-locals-open-p var-tag)
-			   :process process
-                           :object var-value))
-	   ((typep var-value 'jde-dbs-java-primitive)
-	    (widget-create 'tree-widget
-                           :tag var-tag
-                           :node-name var-tag
-                           :open (jde-dbo-locals-open-p var-tag)
-                           :value t
-			   (list 'tree-widget 
-				 :tag (format "%s" (oref var-value value)))))
-	   ((typep var-value 'jde-dbs-java-null)
-	    (widget-create 'tree-widget :tag var-tag :value t
-			   (list 'tree-widget :tag "null")))
-	   (t
-	    (error "Unidentified type of local variable: %s" var-tag)))))
-      (use-local-map widget-keymap)
-      (widget-setup))))
-
+      (dolist (local-var locals)
+        (jde-dbo-view-var-in-buf (oref local-var value)
+                                 (oref local-var name) process
+                                 'jde-dbo-locals-open-p (current-buffer))))))
 
 (defun jde-dbo-update-stack (process thread-id) 
   (let* ((cmd  (jde-dbs-get-thread "get_thread" 
@@ -288,24 +256,10 @@ used in the last breakpoint hit event, and watch point hit event.")
 	      (reason (nth 4 process-state))
 	      (state-info (oref process state-info)))
 	  (if state-info
-	      (progn
-		(jde-dbs-proc-state-info-set state-info state reason
-                                             thread-id thread-name)
-                (setq jde-dbo-current-process process)
-                (setq jde-dbo-current-thread-id thread-id)
-                (if jde-bug-local-variables
-                    (jde-dbo-update-locals-buf process
-                                               thread-id 0))
-                (if jde-bug-stack-info (jde-dbo-update-stack process thread-id))
-		(oset process steppablep t)
-		(jde-dbs-display-debug-message 
-		 process-id
-		 (format "Breakpoint hit at line %d in %s (%s) on thread %s. All threads suspended." 
-			 line-no class file thread-name))
-		(jde-db-set-debug-cursor class file line-no)
-		;; (message (selected-frame))
-		(when jde-bug-raise-frame-p (raise-frame))
-		)
+	      (jde-dbo-break process state-info state reason thread-id
+                             thread-name
+                             (format "Breakpoint hit at line %d in %s (%s) on thread %s. All threads suspended." line-no class file thread-name)
+                             process-id class file line-no)
 	    (message "Breakpoint hit event error: state info object missing for process %d." process-id)))
       (message "Breakpoint hit event error: process object for process %d is missing." process-id))))
 
@@ -322,21 +276,11 @@ used in the last breakpoint hit event, and watch point hit event.")
 	      (reason (nth 4 process-state))
 	      (state-info (oref process state-info)))
 	  (if state-info
-	      (progn
-		(jde-dbs-proc-state-info-set state-info state reason
-                                             thread-id thread-name)
-                (setq jde-dbo-current-process process)
-                (setq jde-dbo-current-thread-id thread-id)
-                (if jde-bug-local-variables
-                    (jde-dbo-update-locals-buf process
-                                               thread-id 0))
-                (if jde-bug-stack-info (jde-dbo-update-stack process thread-id))
-		(oset process steppablep t)
-		(jde-dbs-display-debug-message 
-		 proc-id
-		 (format "Stepped to line %d in %s (%s) on thread %s. All threads suspended." 
-			 line-no class file thread-name))
-		(jde-db-set-debug-cursor class file line-no))
+            (jde-dbo-break process state-info state reason thread-id
+                           thread-name
+                           (format "Stepped to line %d in %s (%s) on thread %s. All threads suspended." 
+                                   line-no class file thread-name)
+                           proc-id class file line-no)
 	    (message "Step event error: state info missing for process %d" proc-id)))
       (message "Step event error: could not find process %d." proc-id))))
 
@@ -350,11 +294,24 @@ used in the last breakpoint hit event, and watch point hit event.")
 	      (thread-name (nth 2 process-state))
 	      (state (nth 3 process-state))
 	      (reason (nth 4 process-state))
+              (location (nth 5 process-state))
 	      (state-info (oref process state-info)))
+          (if (not (equal status "none"))
+            ;; Then it's a break, not a trace
+            (let ((class (nth 1 (car location)))
+                  (file (nth 2 (car location)))
+                  (line-no (nth 3 (car location))))
+              	  (if state-info
+                    (jde-dbo-break process state-info state reason thread-id
+                                   thread-name
+                                   (format "Exception encountered at line %d in %s (%s) on thread %s. All threads suspended." 
+                                           line-no class file thread-name)
+                                   proc-id class file line-no)
+                    (message "Exception event error: state info missing for process %d" proc-id)))
 	  (jde-dbs-display-debug-message 
 	   proc-id
 	   (format "Exception of class %s occurred on thread %s" 
-			 exception-class thread-name))))))
+			 exception-class thread-name)))))))
 
 
 (defun jde-dbo-vm-disconnected-event (process-id process-status thread)
@@ -541,12 +498,85 @@ The remaining elements are arguments to pass to the handler."
 	       (append (list process-id process-status thread) args))))
     events))
 
-
+(defun jde-dbo-view-var-in-buf (var-value name process open buf
+                                          &optional clear)
+  "Create a tree-widget representing variable VAR-VALUE (a
+  jde-dbs-java-null/primitive/udci type), whose name is NAME and in
+  process PROCESS, and place that tree-widget in buffer BUF.  OPEN is
+  either a variable or a function.  If it is a variable, then the
+  tree-widget is open if true.  If it is a function, then the function
+  is called with name to return a true or false value (this is used
+  for caching the state). If CLEAR is true then the buffer is cleared
+  before creating the tree-widget."
+  (save-excursion
+    (set-buffer buf)
+    (when clear
+      (erase-buffer))
+    (let* ((var-tag (format "%s %s [id: %s]" (oref var-value jtype) name
+                           (if (or (typep var-value 'jde-dbs-java-primitive)
+                                    (typep var-value 'jde-dbs-java-null))
+                             "-"
+                             (oref var-value id))))
+          (openp (if (functionp open)
+                   (funcall open var-tag)
+                   open)))
+      (cond
+        ((typep var-value 'jde-dbs-java-udci)
+          (if (string= (oref var-value :jtype) "java.lang.String")
+            (let* ((cmd (jde-dbs-get-string 
+                         "get string"
+                         :process process
+                         :object-id (oref var-value id)))
+                   (str-val (jde-dbs-cmd-exec cmd)))
+              (widget-create 'tree-widget
+                             :tag var-tag
+                             :node-name var-tag
+                             :open openp
+                             :value t
+                             (list 'tree-widget :tag str-val)))
+            (widget-create 'jde-widget-java-obj
+                           :tag var-tag
+                           :node-name var-tag
+                           :open openp
+                           :process process
+                           :object-id (oref var-value :id))))
+	   ((typep var-value 'jde-dbs-java-array)
+	    (widget-create 'jde-widget-java-array
+                           :tag var-tag
+                           :node-name var-tag
+                           :open openp
+			   :process process
+                           :object var-value))
+	   ((typep var-value 'jde-dbs-java-primitive)
+	    (widget-create 'tree-widget
+                           :tag var-tag
+                           :node-name var-tag
+                           :open openp
+                           :value t
+			   (list 'tree-widget 
+				 :tag (format "%s" (oref var-value value)))))
+	   ((typep var-value 'jde-dbs-java-null)
+	    (widget-create 'tree-widget :tag var-tag :value t
+			   (list 'tree-widget :tag "null")))
+	   (t
+	    (error "Unidentified type of variable: %s" var-tag))))
+    (use-local-map widget-keymap)
+    (widget-setup)))
 
 (provide 'jde-dbo)
 ;; $Log: jde-dbo.el,v $
-;; Revision 1.5  2002/09/14 03:20:55  andyp
-;; sync to jde 2.2.9beta12
+;; Revision 1.39  2003/09/27 05:34:36  ahyatt
+;; Exceptions should break when the user selects to suspend the thread as
+;; well (not just when the user selects to suspect all threads).
+;;
+;; Revision 1.38  2003/09/17 05:12:24  ahyatt
+;; Added exception breakpoints
+;;
+;; Revision 1.37  2002/12/19 22:19:06  ahyatt
+;; Fixed problem with nulls in the tree-view display
+;;
+;; Revision 1.36  2002/12/08 20:43:45  ahyatt
+;; Refactoring to support UI changes in jde-bug
 ;;
 ;; Revision 1.35  2002/02/04 05:47:17  paulk
 ;; Added code to rehighlight breakpoints if the user kills a
