@@ -1,11 +1,11 @@
 ;; jde-run.el --- runs the Java app in the current buffer.
-;; $Revision: 1.6 $ $Date: 2002/09/14 03:21:05 $
+;; $Revision: 1.89 $ $Date: 2003/11/18 06:28:19 $
 
 ;; Author: Paul Kinnucan <paulk@mathworks.com>
 ;; Maintainer: Paul Kinnucan
 ;; Keywords: tools, processes
 
-;; Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002 Paul Kinnucan
+;; Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002, 2003 Paul Kinnucan
 
 ;; GNU Emacs is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -200,7 +200,8 @@ Property Value field. You can specify as many properties as you like."
 				     :tag "Sort output based on:"
 		 (const "Allocation objects")
 		 (const "Live objects")))))
-		 
+
+;; (makunbound 'jde-run-option-verify)
 (defcustom jde-run-option-verify (list nil t)
   "*Verify classes."
   :group 'jde-run-options
@@ -210,34 +211,225 @@ Property Value field. You can specify as many properties as you like."
 	       (checkbox :format "%[%v%] %t \n"
 			 :tag "Classes loaded by a classloader.")))
 
-;; (makunbound 'jde-run-option-connect-to-debugger)
-(defcustom jde-run-option-connect-to-debugger (list t "Attach" nil)
-  "*If checked, this option allows the application to start and then
-connect to a debugger, for example, jdb or JDEbug. \"DebuggerConnect
-Mode\" specifies the method for establishing the connection. The
-options are \"Attach\" (allow a debugger to attach to the application
-after the applicaiton starts) or \"Client\" (start the application and
-then try to connect the application to a listening debugger). On Unix
- systems, the \"Address\" option specifies the port number
-of the socket used to connect the debugger and the application. On
-Windows systems, it specifies an arbitrary identifier. If \"Address\"
-is nil, the JDEE uses a default value. The default is \"javadebug\" on
-Windows systems and 4444 on Unix systems."
-  :group 'jde-run-options
-  :type  '(list
-	   (checkbox :format "%[%v%] %t \n\n"
-                     :tag "Enabled")
-	   (radio-button-choice :format "  %t\n%v" 
-				:indent 2
-                                :tag "Debugger Connect Mode:"
-	    (const "Attach")
-	    (const "Listen"))
-	   (choice :format "\n  %[Address%] %v\n\n"
-	    (const :menu-tag "Use Default"
-                   nil)
-	    (string :menu-tag "Specify" :tag "Value"))))
-	  
+;; (makunbound 'jde-run-option-boot-classpath)
+(defcustom jde-run-option-boot-classpath nil
+  "Specify a list of directories, JAR archives, and ZIP archives to
+search for boot class files.
 
+To specify the standard boot classpath, select \"standard\" from the
+Value Menu. To specify a custom boot classpath, select \"custom\" from
+the Value Menu. Emacs displays a \"Custom Mode\" Value Menu and an
+\"INS\" button for creating a list of directories and/or JAR and ZIP
+archives. To append the custom list to the standard list, select
+\"append\" from the Value Menu.  To prepend the custom list to the
+standard list, select \"prepend\" from the Value Menu. To replace the
+standard list with the custom list, select \"replace\" from the Value
+Menu. 
+
+Note that if `jde-jdk' specifies the 1.2.x version of the JDK, the
+JDEE replaces the standard list with the custom list regardless of the
+setting of the \"Custom Mode\" Value Menu. The JDEE ignores this
+option if `jde-jdk' specifies the 1.1.x version of the JDK.
+
+Note also that the value of this variable is either nil or a cons
+whose cdr is a list of strings, each of which specifies a path. The
+JDE converts this list to a colon- or semicolon-separated list before
+inserting it in the compiler or vm command line. The paths may start
+with a tilde (~) and may include environment variables. The JDE
+replaces the ~ with your home directory and replaces each instance of
+an environment variable with its value before inserting it into a
+command line. The paths may also start with a . (period) to indicate
+that the path is a relative path. If `jde-resolve-relative-paths-p' is
+nonnil, the JDEE treats the paths as relative to the location of the
+project file for the current project and replaces the period (.) with
+the path of the directory containing the project file."
+  :group 'jde-run-options
+  :type '(choice 
+	  :tag "Classpath Options"
+	  (const :tag "standard" nil)
+	  (cons :tag "custom" :inline nil
+		(choice 
+		 :tag "Custom Mode"
+		 (const "append")
+		 (const "prepend")
+		 (const "replace"))	   
+		(repeat 
+		 :tag "Classpath" 
+		 (file :tag "Path")))))
+
+
+;; (makunbound 'jde-run-option-debug)
+(defcustom jde-run-option-debug nil
+  "*If \"Connect\" is selected, this option allows the application to
+  start and then connect to a debugger, for example, jdb or JDEbug.
+
+\"Mode\" specifies the method for establishing the connection. The
+options are \"Server\" (allow a debugger to attach to the application
+after the application starts) or \"Client\" (allows the application to connect 
+to a listening debugger).
+
+\"Data Transport\" specifies the method that the debuggee process uses to
+communicate with the debugger. The \"Shared Memory\" option is valid
+only when the the debuggee process and the debugger are both running
+on the same Microsoft Windows system. (It is also the best choice for
+such debugging sessions.) You must choose \"Socket\" if the debugger
+and the debuggee process run on different systems or if they both
+run on the same Unix system.
+
+The \"Shared Memory Name\" option specifies the name of the shared
+memory connection used to connect the debugger and the application.
+This option applies only when you specify a shared memory connection
+to the debugger.
+
+The \"Socket Host\" specifes the host on which a remote debugger
+resides. This option applies only when you run the process in 
+client mode, using a socket transport.
+ 
+The \"Socket Port\" option specifies the socket port used to 
+connect this process to the debugger. This option applies only
+when you select socket as the transport method.
+
+The \"Suspend\" option specifies whether the vm should suspend
+this process on startup."
+  :group 'jde-run-options
+  :type  '(choice 
+	   :tag "Debug Connection Options"
+	   (const :tag "No connect" nil)
+	   (list
+	    :tag "Connect" 
+	    :inline nil
+	    (choice
+	     :tag "Mode"
+	     (const "Server")
+	     (const "Client"))
+	    (choice
+	     :tag "Data Transport"
+	     (const "Shared Memory")
+	     (const "Socket"))
+	    (choice
+	     :tag "Shared Memory Name"
+	     (const :menu-tag "Default" "javadebug")
+	     (string :menu-tag "Custom" :tag "Name"))
+	    (choice	     
+	     :tag "Socket Host"
+	     (const :menu-tag "Local" nil)
+	     (string :menu-tag "Remote" :tag "Name"))
+	    (choice	     
+	     :tag "Socket Port"
+	     (const :menu-tag "Default" "4444")
+	     (string :menu-tag "Custom" :tag "Address"))
+	    (choice
+	     :tag "Suspend?"
+	     (const :tag "No" nil)
+	     (const :tag "Yes" t)))))	  
+
+(defcustom jde-run-option-interpret-mode nil
+  "Causes the vm to interpret all byte codes. By default VMs
+predating JDK 1.3 use a JIT compiler to execute bytecodes. A JIT
+compiler translates the class bytecodes into native machine code when
+a class is loaded. Beginning with JDK 1.3, VMs use a Hotspot
+compiler. A Hotspot compiler compiles sections of code that execute
+frequently."
+   :group 'jde-run-options
+   :type 'boolean)
+
+(defcustom jde-run-option-jar nil
+  "Execute a program encapsulated in a JAR file.
+Set `jde-run-application-class' to the name of a JAR file instead of a
+startup class name. In order for this option to work, the manifest of
+the JAR file must contain a line of the form Main-Class:
+classname. Here, classname identifies the class having the public
+static void main(String[] args) method that serves as your
+application's starting point. See the Jar tool reference page and the
+Jar trail of the Java Tutorial for information about working with Jar
+files and Jar-file manifests.  When you use this option, the JAR file
+is the source of all user classes, and other user class path settings
+are ignored."
+  :group 'jde-run-options
+  :type 'boolean)
+
+
+(defcustom jde-run-option-hotspot-type 'client
+  "Specify whether to use the Hotspot client or server vm."
+  :group 'jde-run-options
+  :type  '(choice :tag "vm type"
+                  (const :tag "client"    client)
+                  (const :tag "server"    server)))
+
+
+;;(makunbound 'jde-run-option-enable-assertions)
+(defcustom jde-run-option-enable-assertions "Nowhere"
+  "Enable assertions for the current project. 
+
+To disable assertions everywhere (the default), select \"Nowhere\"
+from the Value Menu. To enable assertions everywhere except in system
+classes, select \"Everywhere\" from the Value Menu. To enable
+assertions in specific locations, select \"Somewhere\" from the Value
+Menu. The customization buffer expands, allowing you to enable
+assertions in the package in the current directory (the directory of
+the current Java source buffer), and in specific packages and classes.
+
+To enable assertions everwhere except in specified locations, set this
+option to \"Everywhere\" and use `jde-run-option-disable-assertions'
+to specify the exceptions. To enable assertions in system classes, set
+`jde-run-option-enable-system-assertions' on.
+
+The JDEE  ignores this option if `jde-jdk' specifies a version
+of the JDK that precedes version 1.4, which 
+introduced assertions into Java."
+  :group 'jde-run-options
+  :type '(choice :tag "Enable assertions"
+	      (const "Nowhere")
+              (const "Everywhere")
+	      (cons :tag "Somewhere" :inline "Everywhere"
+		    (boolean :tag "In current directory")
+		    (repeat :tag "In the following locations"  
+			    (cons :tag "Location"
+				  (choice :tag "Type"
+					  (const "package")
+					  (const "class"))
+				  (string :tag "Name"))))))
+
+;;(makunbound 'jde-run-option-disable-assertions)
+(defcustom jde-run-option-disable-assertions "Nowhere"
+  "Enable assertions.
+
+Use this option to specify exceptions to packages and classes enabled
+by `jde-run-option-enable-assertions'. To not disable assertions
+anywhere (the default), select \"Nowhere\" from the Value Menu. To
+disable assertions everywhere except in system classes, select
+\"Everywhere except system\" from the Value Menu or set
+`jde-run-option-enable-system-assertions' on.  To disable assertions
+in specific locations, select \"Somewhere\" from the Value Menu. The
+customization buffer expands, allowing you to enable assertions in the
+package in the current directory (the directory of the current Java
+source buffer), and in specific packages and classes.
+
+The JDEE  ignores this option if `jde-jdk' specifies a version
+of the JDK that precedes version 1.4, which 
+introduced assertions into Java."
+  :group 'jde-run-options
+  :type '(choice :tag "Disable assertions"
+	      (const "Nowhere")
+              (const "Everywhere")
+	      (cons :tag "Somewhere" :inline "Everywhere"
+		    (boolean :tag "In current directory")
+		    (repeat :tag "In the following locations"  
+			    (cons :tag "Location"
+				  (choice :tag "Type"
+					  (const "package")
+					  (const "class"))
+				  (string :tag "Name"))))))
+		   
+(defcustom jde-run-option-enable-system-assertions nil
+  "Enable assertions in system classes."
+  :group 'jde-run-options
+  :type 'boolean)
+
+(defcustom jde-run-option-disable-system-assertions nil
+  "Disable assertions in system classes."
+  :group 'jde-run-options
+  :type 'boolean)
 
 (defcustom jde-run-option-vm-args nil
   "*Specify arguments to be passed to the Java vm.
@@ -376,7 +568,7 @@ panel to specifying the applet document."
 		     :type string
 		     :initform ""
 		     :documentation
-		     "Compiler version.")
+		     "Java virtual machine version number.")
    (path             :initarg :path
 		     :type string
 		     :documentation
@@ -413,19 +605,6 @@ panel to specifying the applet document."
 	(list "-classic")))
 
     
-(defmethod jde-run-verbose-arg ((this jde-run-vm))
-  "Set the verbose options."
-    (let ((print-classes-loaded
-	   (nth 0 jde-run-option-verbose))
-	  (print-memory-freed
-	   (nth 1 jde-run-option-verbose))
-	  (print-jni-info
-	   (nth 2 jde-run-option-verbose)))
-      (append
-       (if print-classes-loaded (list "-v"))
-       (if print-memory-freed (list "-verbosegc"))
-       (if print-jni-info (list "-verbosejni")))))
-
 (defmethod jde-run-property-args ((this jde-run-vm))
     "Get property arguments."
     (mapcar
@@ -473,17 +652,6 @@ panel to specifying the applet document."
        (if (not (string= java-size "400k"))
            (list (concat "-Xoss" java-size))))))
 
-(defmethod jde-run-gc-args ((this jde-run-vm))
-  "Get garbage collection arguments."
-    (let ((no-gc-asynch (not 
-			 (nth 0 jde-run-option-garbage-collection)))
-	  (no-gc-classes (not 
-			  (nth 1 jde-run-option-garbage-collection))))
-      (append
-       (if no-gc-asynch
-	  '("-Xnoasyncgc"))
-       (if no-gc-classes
-	   '("-Xnoclassgc")))))
 
 (defmethod jde-run-java-profile-arg ((this jde-run-vm))
    "Get Java profile option."
@@ -514,53 +682,11 @@ panel to specifying the applet document."
 	      "-Xhprof:file=%s,depth=%d,top=%d,sort=%s"
 	      file depth top sort))))))
 
-(defmethod jde-run-verify-args ((this jde-run-vm))
-  "Get verify arguments."
-    (let ((verify-all (nth 0 jde-run-option-verify))
-	  (verify-remote (nth 1 jde-run-option-verify)))
-      (append
-       (if verify-all
-	  '("-Xverify"))
-;;       (if verify-remote
-;;	   '("-Xverifyremote"))
-       (if (and
-	   (not verify-all)
-	   (not verify-remote))
-	   '("-Xnoverify")))))
-
-(defmethod jde-run-debugger-connect-args ((this jde-run-vm))
-  "Get arguments required to allow process to connect
-to a debugger."
-  (if (car jde-run-option-connect-to-debugger)
-      (let ((mode (nth 1 jde-run-option-connect-to-debugger))
-	     (address (nth 2 jde-run-option-connect-to-debugger))
-	     (ms-windows (eq system-type 'windows-nt)))
-	(list "-Xdebug"
-	      (format 
-	       "-Xrunjdwp:transport=%s,address=%s,server=%s,suspend=n"
-	       (if ms-windows "dt_shmem" "dt_socket")
-	       (if address address (if ms-windows "javadebug" "4444"))
-	       (if (string= mode "Attach") "y" "n"))))))
 
 (defmethod jde-run-vm-args ((this jde-run-vm))
   "Get command line args."
   jde-run-option-vm-args)
 
-(defmethod jde-run-get-vm-args ((this jde-run-vm))
-  (append
-   ;; Classic mode argument must come first.
-   (jde-run-classic-mode-arg this)
-   (jde-run-classpath-arg this)
-   (jde-run-verbose-arg this)
-   (jde-run-property-args this)
-   (jde-run-heap-size-args this)
-   (jde-run-stack-size-args this)
-   (jde-run-gc-args this)
-   (jde-run-java-profile-arg this)
-   (jde-run-heap-profile-arg this)
-   (jde-run-verify-args this)
-   (jde-run-debugger-connect-args this)
-   (jde-run-vm-args this)))
 
 (defmethod jde-run-vm-launch ((this jde-run-vm))
   (let ((run-buf-name (concat "*" (oref this :main-class) "*"))
@@ -616,49 +742,362 @@ to a debugger."
       (message "An instance of %s is running." (oref this :main-class))
       (pop-to-buffer run-buf-name))))
 
-;;;###autoload
-(defun jde-run (main-class)
-  "Run the Java application specified by `jde-run-executable', if
-not the null string. Otherwise run the class specified by 
-`jde-run-application-class', if non-null; otherwise the class in
-the current buffer. Specifying a prefix argument, e.g.,
-C-u C-c C-v C-r, causes this command to prompt you to enter
-the name of the application's main class This command 
-creates a comint buffer to allow you to interact with the program."
-  (interactive "P")
-  (if (equal major-mode 'jde-mode)
-      (if (string= jde-run-executable "")
-	  (let ((vm (jde-run-get-vm)))
-	    (oset
-	     vm
-	     :main-class
-	     (if main-class
-		 (read-from-minibuffer
-		  "Main class: "
-		  (concat (jde-db-get-package)
-		      (file-name-sans-extension 
-		       (file-name-nondirectory (buffer-file-name)))))
-	       (jde-run-get-main-class)))
-	    (jde-run-vm-launch vm))
-	(jde-run-executable))
-    (error "The jde-run command works only in a Java source buffer.")))
 
-(defun jde-run-get-main-class () 
-  "Gets the main class for the application to which the current
-source buffer belongs."
-  (let ((main-class jde-run-application-class))
-    (if (or
-	 (not main-class)
-	 (string= main-class ""))
-	(setq main-class
-	      (concat (jde-db-get-package)
-		      (file-name-sans-extension 
-		       (file-name-nondirectory (buffer-file-name))))))
-    main-class))
+(defclass jde-run-vm-1-1 (jde-run-vm) ()
+  "Represents the JDK 1.1.x vm")
+
+(defmethod initialize-instance ((this jde-run-vm-1-1) &rest fields)
+  "Constructor for the class representing the JDK 1.1 vm."
+
+  ;; Call parent initializer.
+  (call-next-method)
+  
+  (oset this :version "1.1"))
+
+(defmethod jde-run-verbose-arg ((this jde-run-vm-1-1))
+  "Set the verbose options."
+    (let ((print-classes-loaded
+	   (nth 0 jde-run-option-verbose))
+	  (print-memory-freed
+	   (nth 1 jde-run-option-verbose))
+	  (print-jni-info
+	   (nth 2 jde-run-option-verbose)))
+      (append
+       (if print-classes-loaded (list "-verbose"))
+       (if print-memory-freed (list "-verbosegc"))
+       (if print-jni-info (list "-verbose")))))
+
+(defmethod jde-run-gc-args ((this jde-run-vm-1-1))
+  "Get garbage collection arguments."
+    (let ((no-gc-asynch (not 
+			 (nth 0 jde-run-option-garbage-collection)))
+	  (no-gc-classes (not 
+			  (nth 1 jde-run-option-garbage-collection))))
+      (append
+       (if no-gc-asynch
+	  '("-noasyncgc"))
+       (if no-gc-classes
+	   '("-noclassgc")))))
+
+(defmethod jde-run-verify-args ((this jde-run-vm-1-1))
+  "Get verify arguments."
+    (let ((verify-all (nth 0 jde-run-option-verify))
+	  (verify-remote (nth 1 jde-run-option-verify)))
+      (append
+       (if verify-all
+	  '("-verify"))
+       (if (and
+	   (not verify-all)
+	   (not verify-remote))
+	   '("-noverify")))))
+
+(defmethod jde-run-debug-args ((this jde-run-vm-1-1))
+  "Get arguments required to allow process to connect
+to a debugger."
+  (if jde-run-option-debug
+      '("-debug")))
+
+(defmethod jde-run-interpret-mode-arg ((this jde-run-vm-1-1))
+  "Get argument required to enable interpret mode."
+  (if jde-run-option-interpret-mode
+      '("-nojit")))
+
+
+(defmethod jde-run-get-vm-args ((this jde-run-vm-1-1))
+  (append
+   (jde-run-classpath-arg this)
+   (jde-run-verbose-arg this)
+   (jde-run-property-args this)
+   (jde-run-heap-size-args this)
+   (jde-run-stack-size-args this)
+   (jde-run-gc-args this)
+   (jde-run-java-profile-arg this)
+   (jde-run-heap-profile-arg this)
+   (jde-run-verify-args this)
+   (jde-run-interpret-mode-arg this)
+   (jde-run-debug-args this)
+   (jde-run-vm-args this)))
+
+
+(defclass jde-run-vm-1-2 (jde-run-vm-1-1) ()
+  "Represents the JDK 1.2.x vm")
+
+(defmethod initialize-instance ((this jde-run-vm-1-2) &rest fields)
+  "Constructor for the class representing the JDK 1.2 vm."
+
+  ;; Call parent initializer.
+  (call-next-method)
+  
+  (oset this :version "1.2"))
+
+(defmethod jde-run-boot-classpath-arg ((this jde-run-vm-1-2))
+  "Returns the boot classpath argument for this vm."
+  (if jde-run-option-boot-classpath
+	(list
+	 (concat
+	  "-Xbootclasspath:"
+	  (jde-build-classpath
+	   (cdr jde-run-option-boot-classpath) 
+	   'jde-run-option-boot-classpath)))))
+
+(defmethod jde-run-verbose-arg ((this jde-run-vm-1-2))
+  "Set the verbose options."
+    (let ((print-classes-loaded
+	   (nth 0 jde-run-option-verbose))
+	  (print-memory-freed
+	   (nth 1 jde-run-option-verbose))
+	  (print-jni-info
+	   (nth 2 jde-run-option-verbose)))
+      (append
+       (if print-classes-loaded (list "-verbose:class"))
+       (if print-memory-freed (list "-verbose:gc"))
+       (if print-jni-info (list "-verbose:jni")))))
+
+(defmethod jde-run-gc-args ((this jde-run-vm-1-2))
+  "Get garbage collection arguments."
+    (let ((no-gc-asynch (not 
+			 (nth 0 jde-run-option-garbage-collection)))
+	  (no-gc-classes (not 
+			  (nth 1 jde-run-option-garbage-collection))))
+      (append
+       (if no-gc-asynch
+	  '("-Xnoasyncgc"))
+       (if no-gc-classes
+	   '("-Xnoclassgc")))))
+
+(defmethod jde-run-debug-args ((this jde-run-vm-1-2))
+  "Get arguments required to allow process to connect
+to a debugger."
+  (if jde-run-option-debug
+      '("-Xdebug")))
+
+(defmethod jde-run-interpret-mode-arg ((this jde-run-vm-1-2))
+  "Get argument required to disable use of JIT compiler."
+  (if jde-run-option-interpret-mode
+      '("-Djava.compiler=NONE")))
+
+(defmethod jde-run-jar-arg ((this jde-run-vm-1-2))
+  "Get argument that specifies use of a jar file to run an app."
+  (if (car jde-run-option-jar)
+      '("-jar")))
+
+
+(defmethod jde-run-get-vm-args ((this jde-run-vm-1-2))
+  (append
+   ;; Classic mode argument must come first.
+   (jde-run-classic-mode-arg this)
+   (jde-run-boot-classpath-arg this)
+   (jde-run-classpath-arg this)
+   (jde-run-verbose-arg this)
+   (jde-run-property-args this)
+   (jde-run-heap-size-args this)
+   (jde-run-stack-size-args this)
+   (jde-run-gc-args this)
+   (jde-run-java-profile-arg this)
+   (jde-run-heap-profile-arg this)
+   (jde-run-debug-args this)
+   (jde-run-interpret-mode-arg this)
+   (jde-run-vm-args this)
+   (jde-run-jar-arg this) ;; must be last
+   ))
+
+(defclass jde-run-vm-1-3 (jde-run-vm-1-2) ()
+  "Represents the JDK 1.3.x vm")
+
+(defmethod initialize-instance ((this jde-run-vm-1-3) &rest fields)
+  "Constructor for the class representing the JDK 1.3 vm."
+
+  ;; Call parent initializer.
+  (call-next-method)
+  
+  (oset this :version "1.3"))
+
+
+(defmethod jde-run-boot-classpath-arg ((this jde-run-vm-1-3))
+  "Returns the boot classpath argument for this vm."
+  (if jde-run-option-boot-classpath
+      (list
+       (concat 
+	"-Xbootclasspath"
+	(let ((mode (car jde-run-option-boot-classpath)))
+	  (cond
+	   ((string= mode "append")
+	    "/a:")
+	   ((string= mode "prepend")
+	    "/p:")
+	   ((string= mode "replace")
+	    ":")
+	   (t
+	    (error "Illegal custom classpath mode: %s"  mode))))
+	(jde-build-classpath
+	 (cdr jde-run-option-boot-classpath) 
+	 'jde-run-option-boot-classpath)))))
+
+
+(defmethod jde-run-debug-args ((this jde-run-vm-1-3))
+  "Get arguments required to allow process to connect
+to a debugger."
+  (if jde-run-option-debug
+      (let ((mode (nth 0 jde-run-option-debug))
+	    (transport (nth 1 jde-run-option-debug))
+	    (shared-mem-name (nth 2 jde-run-option-debug))
+	    (socket-host (nth 3 jde-run-option-debug))
+	    (socket-port (nth 4 jde-run-option-debug))
+	    (suspend (nth 5 jde-run-option-debug))
+	    (ms-windows (eq system-type 'windows-nt)))
+	(list "-Xdebug"
+	      (format 
+	       "-Xrunjdwp:transport=%s,address=%s,server=%s,suspend=%s"
+	       (if (string= transport "Shared Memory")
+		   (if ms-windows
+		       "dt_shmem"
+		     (error "Shared memory transport is valid only on Windows."))
+		 "dt_socket")
+	       (if  (string= transport "Shared Memory") 
+		   shared-mem-name
+		 (if (string= mode "Client")
+		     (if socket-host
+			 (concat socket-host ":" socket-port)
+		       socket-port)
+		   socket-port))		       
+	       (if (string= mode "Server") "y" "n")
+	       (if suspend "y" "n"))))))
+
+(defmethod jde-run-get-vm-args ((this jde-run-vm-1-3))
+  (append
+   ;; Classic mode argument must come first.
+   (jde-run-classic-mode-arg this)
+   (jde-run-boot-classpath-arg this)
+   (jde-run-classpath-arg this)
+   (jde-run-verbose-arg this)
+   (jde-run-property-args this)
+   (jde-run-heap-size-args this)
+   (jde-run-stack-size-args this)
+   (jde-run-gc-args this)
+   (jde-run-java-profile-arg this)
+   (jde-run-heap-profile-arg this)
+   (jde-run-debug-args this)
+   (jde-run-interpret-mode-arg this)
+   (jde-run-vm-args this)
+   (jde-run-jar-arg this) ;; must be last
+   ))
+
+
+(defclass jde-run-vm-1-4 (jde-run-vm-1-3) ()
+  "Represents the JDK 1.4.x vm")
+
+(defmethod initialize-instance ((this jde-run-vm-1-4) &rest fields)
+  "Constructor for the class representing the JDK 1.4 vm."
+
+  ;; Call parent initializer.
+  (call-next-method)
+  
+  (oset this :version "1.4"))
+
+
+(defmethod jde-run-hotspot-vm-type-arg ((this jde-run-vm-1-4))
+  "Get the vm argument that specifes the Hotspot vm type."
+  (if (eq jde-run-option-hotspot-type 'server)
+      '("-server")))
+
+(defmethod jde-run-interpret-mode-arg ((this jde-run-vm-1-4))
+  "Get argument required to enable interpret mode."
+  (if jde-run-option-interpret-mode
+      '("-Xint")))
+
+(defmethod jde-run-enable-assertions-args ((this jde-run-vm-1-4))
+  "Get argument(s) required to enable assertions."
+  (let (args)
+    (cond
+     ((stringp jde-run-option-enable-assertions)
+      (cond
+       ((string= jde-run-option-enable-assertions "Nowhere"))
+       ((string= jde-run-option-enable-assertions "Everywhere")
+	(setq args '("-ea")))
+       (t
+	(error "Illegal enable assertions option: \"%s\"." 
+	       jde-run-option-enable-assertions))))
+     ((listp jde-run-option-enable-assertions)
+      (if (car jde-run-option-enable-assertions)
+	  (setq args '("-ea:...")))
+      (loop for location in (cdr jde-run-option-enable-assertions) do
+	    (let ((type (car location))
+		  (name (cdr location)))
+	      (if (string= type "package")
+		  (setq name (concat name "...")))
+	      (setq args  (append args (list (concat "-ea:" name)))))))
+     (t
+      (error "Illegal enable assertions option: \"%s\"." 
+		jde-run-option-enable-assertions)))
+    args))
+
+
+(defmethod jde-run-disable-assertions-args ((this jde-run-vm-1-4))
+  "Get argument(s) required to enable assertions."
+  (let (args)
+    (cond
+     ((stringp jde-run-option-disable-assertions)
+      (cond
+       ((string= jde-run-option-disable-assertions "Nowhere"))
+       ((string= jde-run-option-disable-assertions "Everywhere")
+	(setq args '("-da")))
+       (t
+	(error "Illegal disable assertions option: \"%s\"." 
+	       jde-run-option-disable-assertions))))
+     ((listp jde-run-option-disable-assertions)
+      (if (car jde-run-option-disable-assertions)
+	  (setq args '("-da:...")))
+      (loop for location in (cdr jde-run-option-disable-assertions) do
+	    (let ((type (car location))
+		  (name (cdr location)))
+	      (if (string= type "package")
+		  (setq name (concat name "...")))
+	      (setq args  (append args (list (concat "-da:" name)))))))
+     (t
+      (error "Illegal disable assertions option: \"%s\"." 
+		jde-run-option-disable-assertions)))
+    args))
+
+			
+(defmethod jde-run-enable-system-assertions-arg ((this jde-run-vm-1-4))
+  "Get argument required to enable system assertions."
+  (if jde-run-option-enable-system-assertions
+      '("-esa")))
+
+(defmethod jde-run-disable-system-assertions-arg ((this jde-run-vm-1-4))
+  "Get argument required to disable system assertions."
+  (if jde-run-option-disable-system-assertions
+      '("-dsa")))
+
+(defmethod jde-run-get-vm-args ((this jde-run-vm-1-4))
+  (append
+   ;; Classic mode argument must come first.
+   (jde-run-classic-mode-arg this)
+   (jde-run-boot-classpath-arg this)
+   (jde-run-classpath-arg this)
+   (jde-run-verbose-arg this)
+   (jde-run-property-args this)
+   (jde-run-heap-size-args this)
+   (jde-run-stack-size-args this)
+   (jde-run-gc-args this)
+   (jde-run-java-profile-arg this)
+   (jde-run-heap-profile-arg this)
+   (jde-run-debug-args this)
+   (jde-run-interpret-mode-arg this)
+   (jde-run-enable-assertions-args this)
+   (jde-run-disable-assertions-args this)
+   (jde-run-enable-system-assertions-arg this)
+   (jde-run-disable-system-assertions-arg this)
+   (jde-run-vm-args this)
+   (jde-run-jar-arg this) ;; must be last
+   ))
 
 (defvar jde-run-virtual-machines
   (list
-   (jde-run-vm "JDK vm"))
+   (jde-run-vm-1-1 "JDK 1.1 vm")
+   (jde-run-vm-1-2 "JDK 1.2 vm")
+   (jde-run-vm-1-3 "JDK 1.3 vm")
+   (jde-run-vm-1-4 "JDK 1.4 vm"))
   "*List of supported virtual machines.")
 
 (defun jde-run-get-vm ()
@@ -678,35 +1117,72 @@ source buffer belongs."
 	  (let ((vm-path 
 		 (substitute-in-file-name jde-vm-path)))
 	    (if (string= vm-path "")
-		(let ((jdk-dir (jde-get-jdk-dir)))
-		  (if jdk-dir
-		      (progn
-			 (setq vm-path 
-			       (expand-file-name 
-				(if (eq system-type 'windows-nt) 
-				    "bin/javaw.exe" 
-				  "bin/java") 
-				jdk-dir))
-			 (if (file-exists-p vm-path)
-			     vm-path
-			   (error (format "Invalid vm path %s" 
-					  vm-path))))
-		       (if (executable-find "java")
-			   (if (eq system-type 'windows-nt) "javaw" "java")
-			 (error "Cannot find a Java vm."))))
-		  (if (file-exists-p
-		       (if (and
-			    (eq system-type 'windows-nt)
-			    (not (string-match "[.]exe$" vm-path)))
-			   (concat vm-path ".exe")
-			 vm-path))
-		      vm-path
-		    (if (executable-find vm-path)
-			vm-path
-		      (error "Invalid vm path: %s"
-			     vm-path))))))
+		(jde-get-jdk-prog (if (eq system-type 'windows-nt)
+				      'javaw 'java))
+	      (if (file-exists-p
+		   (if (and
+			(eq system-type 'windows-nt)
+			(not (string-match "[.]exe$" vm-path)))
+		       (concat vm-path ".exe")
+		     vm-path))
+		  vm-path
+		(if (executable-find vm-path)
+		    vm-path
+		  (error "Invalid vm path: %s"
+			 vm-path))))))
     vm))
 
+;;;###autoload
+(defun jde-run (prefix)
+  "Run the Java application specified by `jde-run-executable', if
+not the null string. Otherwise run the class specified by
+`jde-run-application-class', if non-null; otherwise the class in the
+current buffer. Specifying a prefix argument, e.g., C-u C-c C-v C-r,
+causes this command to prompt you to enter arguments to be passed to
+the application's main method.  Specifying a minus prefix argument,
+i.e., C-u - C-c C-v C-r, causes this command to prompt you to also
+enter the name of the application's main class.  Note that you can use
+`jde-run-read-app-args' to cause the command to prompt you for
+application arguments by default (i.e., without having to specify a
+prefix argument). This command creates a comint buffer to allow you to
+interact with the program."
+  (interactive "p")
+  (if (equal major-mode 'jde-mode)
+      (if (string= jde-run-executable "")
+	  (let ((vm (jde-run-get-vm))
+		(read-app-args
+		 (or jde-run-read-app-args
+		     (not (= prefix 1))))
+		(read-main-class
+		  (= prefix -1)))
+	    (oset
+	     vm
+	     :main-class
+	     (if read-main-class
+		 (read-from-minibuffer
+		  "Main class: "
+		  (concat (jde-db-get-package)
+		      (file-name-sans-extension 
+		       (file-name-nondirectory (buffer-file-name)))))
+	       (jde-run-get-main-class)))
+	    (let ((jde-run-read-app-args read-app-args))
+	      (jde-run-vm-launch vm)))
+	(jde-run-executable))
+    (error "The jde-run command works only in a Java source buffer.")))
+
+(defun jde-run-get-main-class () 
+  "Gets the main class for the application to which the current
+source buffer belongs."
+  (let ((main-class jde-run-application-class))
+    (if (or
+	 (not main-class)
+	 (string= main-class ""))
+	(setq main-class
+	      (concat (jde-db-get-package)
+		      (file-name-sans-extension 
+		       (file-name-nondirectory (buffer-file-name))))))
+    main-class))
+
 (defun jde-run-main-class()
   "Runs the Java program named by `jde-run-application-class' in
 a buffer, piping output from the program to the buffer and 
@@ -716,7 +1192,6 @@ input from the buffer to the program."
     (oset vm :main-class (jde-run-get-main-class))
     (jde-run-vm-launch vm)))
 
-
 (defmacro save-w32-show-window (&rest body)
   "Saves the value of the w32-start-process-show-window variable
 before evaluating body and restores the value afterwards."
@@ -1046,10 +1521,14 @@ Here goes all the error message parsing."
   "Jump to the stack position at the mouse click.
 Click anywhere on the line with the stack reference."
   (interactive "e")
-  (let ((pos (event-start event)))
-    (set-marker (car jde-run-etrace-current-marker)
-                (posn-point pos)
-                (window-buffer (posn-window pos))))
+  (if jde-xemacsp
+      (set-marker (car jde-run-etrace-current-marker)
+		  (event-point pos)
+		  (window-buffer (event-window pos)))
+    (let ((pos (event-start event)))
+      (set-marker (car jde-run-etrace-current-marker)
+		  (posn-point pos)
+		  (window-buffer (posn-window pos)))))
   (jde-run-etrace-goto))
 
 
@@ -1106,8 +1585,67 @@ The point should be anywhere on the line with the stack reference."
 
 ;; Change History
 ;; $Log: jde-run.el,v $
-;; Revision 1.6  2002/09/14 03:21:05  andyp
-;; sync to jde 2.2.9beta12
+;; Revision 1.89  2003/11/18 06:28:19  paulk
+;; Fix the jde-run command so that it prompts you for application arguments if jde-run-read-app-args is nonnil or if the jde-run command has a prefix argument.
+;;
+;; Revision 1.88  2003/09/18 05:35:29  paulk
+;; Fixed bug.
+;;
+;; Revision 1.87  2003/09/14 04:43:19  paulk
+;; Changed jde-run-option-debug to provide separate options for shared memory
+;; name, socket host, and socket port and to specify defaults explicitly.
+;;
+;; Revision 1.86  2003/07/18 12:49:02  paulk
+;; Prefixing the jde-run command (e.g., C-u C-c C-v C-r) now
+;; causes the command to  prompt you to enter application arguments.
+;; Prefixing the command with a - (e.g., C-u - C-c C-v C-r)
+;; causes the command to prompt you to enter the name of
+;; the application's main class as well as application arguments.
+;; In previous releases, prefixing the command caused it only
+;; to prompt for the application main class.
+;;
+;; Revision 1.85  2003/06/15 04:36:56  paulk
+;; Enhance jde-run-option-debug to support suspend option.
+;;
+;; Revision 1.84  2003/06/12 04:55:06  paulk
+;; Change terminology in jde-run-option-debug to match documentation, i.e., Attach becomes Server mode and Listen becomes Client mode.
+;;
+;; Revision 1.83  2003/05/14 06:44:37  paulk
+;; Corrected and expanded the doc string for jde-run-option-debug.
+;;
+;; Revision 1.82  2003/05/14 05:02:02  paulk
+;; Enhance jde-run-option-debug to permit specification of type of data transport
+;; to be used between debugger and debuggee process.
+;;
+;; Revision 1.81  2003/03/28 05:33:30  andyp
+;; XEmacs optimizations for JDEbug and efc.
+;;
+;; Revision 1.80  2003/03/10 06:13:53  paulk
+;; Simplified data type of jde-run-option-debug and set it off by default.
+;;
+;; Revision 1.79  2003/01/08 21:40:13  jslopez
+;; Removes duplicate definition of jde-run-get-vm.
+;;
+;; Revision 1.78  2002/11/30 02:54:22  paulk
+;; Update jde-run-option-boot-classpath to support the append and prepend
+;; options intoduced by JDK 1.3.
+;;
+;; Revision 1.77  2002/11/30 01:20:44  paulk
+;; Added the following customization variables:
+;;  - jde-run-option-disable-assertions
+;;  - jde-run-option-enable-system-assertions
+;;  - jde-run-option-disable-system-assertions
+;;
+;; Revision 1.76  2002/11/29 07:23:48  paulk
+;; Adds the jde-run-option-enable-assertions variable for enabling
+;; assertions.
+;;
+;; Revision 1.75  2002/11/05 08:02:21  paulk
+;; Mac OS X (darwin) compatibility fix: Find path of java on Mac. This is part of a more general
+;; fix that supercedes the one supplied by Andi Vajda. Thanks to Andrew Hyatt.
+;;
+;; Revision 1.74  2002/10/31 05:20:33  paulk
+;; Applied compatibility fix for Mac OS X. Thanks to Andi Vajda <avajda@nanospace.com>.
 ;;
 ;; Revision 1.73  2002/09/10 13:51:47  jslopez
 ;; Fixes regression bug in jde-run-etrace-set-font-lock.
